/*****************************************************************************/
/**                                                                         **/
/** (C) Copyright 2006 - Analog Devices, Inc.  All rights reserved.         **/
/**                                                                         **/
/*****************************************************************************/
/*****************************************************************************/

/*****************************************************************************/
/*               ADI DSP Division, EMP, Munich					             */
/*****************************************************************************/
/*
 *
 * FILE NAME	: I2C_BF5xx_ASM_C_API.c
 *
 * DATE         : $Date: 2006/12/01 10:01:34 $
 *
 * AUTHOR(S)    : $Author: ANALOG\tlorenz $
 *
 * REVISION NO. : $Revision: 0.6 $
 *
 * 
 * DESCRIPTION  :  I2C Protocol State Machine
 *
 *
 * SOFTWARE		: VisualDSP++4.5 (November Update)
 *
 * HARDWARE		: ADSP-BF561 EZ-KIT LITE REV 2.0 ADSP-BF561 Chip REV 0.5
 *				: ADSP-BF533 EZ-KIT LITE REV 1.7 ADSP-BF533 Chip REV 0.4
 *
 *
 *****************************************************************************/
 /******************************************************************************
*   8 / 16 - bit read write routines making use of I2C_BF5xx.asm routines     *
******************************************************************************/

/******************************************************************************

function will read/write a vector of 8 or 16-bit values from/to a device, starting from an address.

Parameters:
	addr_size_16:		false if addresses are 8-bit wide
						true if addresses are 16-bit wide

	data_size_16:		false if data values are 8-bit wide
						true if data values are 16-bit wide

	TWIBase_Addr:		base address (8-bit)
	                    0x40 (ADV7183A/B), 0x54 (ADV7171/ADV7179), 
	                    0xAA (KAC-9618), 0xB8 (MT9V022), 0xC0 (OV6630)

	start_address:		register address of first value in vector

	values:				pointer to first value in vector (vector is always 16-bit, in case of 8-bit values, use only lower bytes)

	Num_Transactions:	Number of values to be written/read

Return Value:

	Receive data can be fetched from SCCB_DataOut[i]
	Error can be tested via "SCCB_Error"
	
******************************************************************************/
#include "I2C_BF5xx_ASM_C_API.h"

unsigned int I2C_Blocked_Write(bool addr_size_16, bool data_size_16, unsigned char TWIBase_Addr, unsigned short start_address, unsigned short* values, int Num_Transactions)
{
    unsigned int num_successful_transactions = 0;
	int i,j;
	
	unsigned int wr_addr = TWIBase_Addr;
	
	for (i=0; i<Num_Transactions; i++)
	{
	    j=0;
		_SCCB_DataIn[j++] = wr_addr;								// base address
		if (addr_size_16) _SCCB_DataIn[j++] = start_address >>8 ;	// high byte start address if applicable
		_SCCB_DataIn[j++] = (start_address++) & 0xFF;				// low byte start address
		if (data_size_16) _SCCB_DataIn[j++] = *(values + i) >> 8 ;	// high byte value if applicable
		_SCCB_DataIn[j++] = *(values + i) & 0xFF;					// low byte value
		
		_SCCB_Word_Count = j;
		_SCCB_Read_Count = 2;
		
		_SCCB_Control = 2;
		_SCCB_Error = 0;
		_SCCB_In_Progress = 1;
		
		_SCCB_Interface();
		
		while(_SCCB_In_Progress){};
		
		if (_SCCB_Error != 0)
	 		break;		// error

		num_successful_transactions++;
		
	}
	return num_successful_transactions;
}


void I2C_NonBlocked_Write(bool addr_size_16, bool data_size_16, unsigned char TWIBase_Addr, unsigned short start_address, unsigned short* values, int Num_Transactions){
	int i,j;

	unsigned int wr_addr = TWIBase_Addr;

	j=0;
	_SCCB_DataIn[j++] = wr_addr;									// base address
	if (addr_size_16) _SCCB_DataIn[j++] = start_address >>8;		// high byte start address if applicable
	_SCCB_DataIn[j++] = (start_address++) & 0xFF;					// low byte start address
	
	for (i=0; i<Num_Transactions; i++){
		if (data_size_16) _SCCB_DataIn[j++] = *(values + i) >> 8;	// high byte value if applicable
		_SCCB_DataIn[j++] = *(values + i) & 0xFF;					// low byte value
	}
	
	_SCCB_Word_Count = j;											// # of values to write
	_SCCB_Read_Count = 2;
	_SCCB_Control = 2;												// write indicator
	_SCCB_Error = 0;												// reset error
	_SCCB_In_Progress = 1;											// tx still in progress indicator
	
	_SCCB_Interface();												// call of SCCB protocol
	
}


unsigned int I2C_Blocked_Read(bool addr_size_16, bool data_size_16, unsigned char TWIBase_Addr, unsigned short start_address, unsigned short* values, int Num_Transactions)
{
    unsigned int num_successful_transactions = 0;
	int i,j;
	
	unsigned int wr_addr = TWIBase_Addr;
	unsigned int rd_addr = wr_addr +1;
	
	j=0;
	_SCCB_DataIn[j++] = wr_addr;								// base address
	if (addr_size_16) _SCCB_DataIn[j++] = start_address >>8 ;	// high byte start address if applicable
	_SCCB_DataIn[j++] = (start_address++) & 0xFF;				// low byte start address
	_SCCB_DataIn[j++] = rd_addr;								// base address
	
	
	_SCCB_Word_Count = j + (1 + (int)data_size_16) * Num_Transactions;
	_SCCB_Read_Count = 2 + (int)addr_size_16;

	_SCCB_Control = 1;
	_SCCB_Error = 0;
	_SCCB_In_Progress = 1;

	_SCCB_Interface();

	while(_SCCB_In_Progress){};
	
	if (_SCCB_Error == 0)
	{
	    num_successful_transactions = Num_Transactions; 	// no error
		for (i=0; i<Num_Transactions; i++)
		    if (data_size_16)
				*(values + i) = (_SCCB_DataOut[2*i + 1] & 0xFF) + (_SCCB_DataOut[2*i + 0] << 8);
			else
				*(values + i) = (_SCCB_DataOut[i] & 0xFF);
	}
	else
		num_successful_transactions = 0;					// error	

	return num_successful_transactions;
}


void I2C_NonBlocked_Read(bool addr_size_16, bool data_size_16, unsigned char TWIBase_Addr, unsigned short start_address, int Num_Transactions)
{
	int i,j;
	
	unsigned int wr_addr = TWIBase_Addr;
	unsigned int rd_addr = wr_addr +1;
	
	j=0;
	_SCCB_DataIn[j++] = wr_addr;										// base address
	if (addr_size_16) _SCCB_DataIn[j++] = start_address >>8;			// high byte start address if applicable
	_SCCB_DataIn[j++] = (start_address++) & 0xFF;						// low byte start address
	_SCCB_DataIn[j++] = rd_addr;										// base address
	
	
	_SCCB_Word_Count = j + (1 + (int)data_size_16) * Num_Transactions;	// # of values to read
	_SCCB_Read_Count = 2 + (int)addr_size_16;

	_SCCB_Control = 1;													// read indicator
	_SCCB_Error = 0;													// reset error
	_SCCB_In_Progress = 1;												// rx still in progress indicator
	
	_SCCB_Interface();													// call of SCCB protocol
	
}
